import threading
import subprocess
import time
import json
import logging

class RemoteAccessHandler:
    def __init__(self, device_client_instance):
        self.device_client = device_client_instance
        self.tunnel = None
        global logger
        logger = logging.getLogger('myur.daemon.' + __name__)

    def create_vpn_tunnel(self, correlation_id, requester, one_time_password, domain):
        self.tunnel = threading.Thread(target=self._start_ngrok_tunnel, args=(requester, one_time_password, domain))
        self.tunnel.daemon = True
        self.tunnel.start()

        url = None
        counter=0

        while url is None and counter < 15:
            time.sleep(1)
            url = self.check_for_url()
            counter=counter+1

        logger.debug ("URL is " + str(url))

        if url is None:
            self.close_ngrok_tunnel(str(correlation_id))
            raise Exception("Ngrok not started yet")

        self._send_message_to_hub('{ "type" : "VncAccess", "messageType" : "VncTunnelCreated", "correlationId" : "'+ str(correlation_id) +'", "SessionUrl": "'+ str(url) +'" }')

    def check_for_url(self):

        try:
            byte_output = subprocess.check_output(["curl", 'localhost:4040/api/tunnels'])
            output = byte_output.decode('UTF-8').rstrip()

            if "ERROR" in output:
                raise Exception("Ngrok not started")

            json_data = json.loads(str(output))
            logger.debug(json_data["tunnels"][0]["public_url"])

            return str(json_data["tunnels"][0]["public_url"]) + "/vnc_lite.html"
        except Exception as ex:
            logger.error (str(ex))
            return None



    def send_remote_request_access_response(self, correlation_id, accepted):
        self._send_message_to_hub('{ "type" : "VncAccess", "messageType" : "VncRequestUserResponse", "correlationId" : "'+ str(correlation_id) +'", "AccessRequestAccepted": "'+ str(accepted) + '" }')

    def send_remote_access_client_connected_event(self, correlation_id):
        self._send_message_to_hub('{ "type" : "VncAccess", "messageType" : "VncClientConnected", "correlationId" : "'+ str(correlation_id) + '" }')


    def _start_ngrok_tunnel(self, requester, one_time_password, domain):
        authArgs=' --basic-auth='+ str(requester) +':'+ str(one_time_password)
        domainArgs=' --domain='+ str(domain)
        cmd = 'ngrok http 6080 --config /root/.config/ngrok/ngrok.yml' + authArgs + domainArgs + '  > /dev/null &'
        logger.debug (cmd)
        subprocess.call(cmd, stderr=subprocess.STDOUT, shell=True)



    def close_ngrok_tunnel(self, correlation_id):
        try:
            subprocess.call('killall ngrok', shell=True)
            self._send_message_to_hub('{ "type" : "VncAccess", "messageType" : "VncTunnelClosed", "correlationId" : "'+ str(correlation_id) +'" }')
        except subprocess.CalledProcessError as e:
            logger.error(str(e))

    def _send_message_to_hub(self, message):
        try:
            self.device_client.send_message(message)
            return True
        except Exception as e:
            logger.error("Failed to send message : " + message + " to hub :" + e)
            return False

